package com.avides.vaadin.spring.example;

import javax.annotation.PostConstruct;

import org.springframework.beans.factory.annotation.Autowired;

import com.avides.vaadin.spring.MessageBean;
import com.vaadin.flow.component.Tag;
import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.dependency.HtmlImport;
import com.vaadin.flow.component.html.Div;
import com.vaadin.flow.component.page.Push;
import com.vaadin.flow.component.polymertemplate.Id;
import com.vaadin.flow.component.polymertemplate.PolymerTemplate;
import com.vaadin.flow.router.Route;
import com.vaadin.flow.shared.ui.Transport;
import com.vaadin.flow.templatemodel.TemplateModel;
import com.vaadin.flow.theme.Theme;
import com.vaadin.flow.theme.lumo.Lumo;

/**
 * Simple template example.
 */
@Route("")
@Theme(Lumo.class)
@Push(transport = Transport.LONG_POLLING)
@Tag("example-template")
@HtmlImport("src/example-template.html")
public class ExampleTemplate extends PolymerTemplate<ExampleTemplate.ExampleModel> implements ExampleView
{
    @Autowired
    private transient ExamplePresenter examplePresenter;

    @Id("someActionButton")
    private Button someActionButton;

    @Id("container")
    private Div container;

    /**
     * Template model which defines the single "value" property.
     */
    public interface ExampleModel extends TemplateModel
    {
        void setValue(String name);
    }

    public ExampleTemplate(@Autowired MessageBean bean)
    {
        // Set the initial value to the "value" property.
        getModel().setValue(bean.getMessage());

        initSomeActionButton();
    }

    @PostConstruct
    public void init()
    {
        examplePresenter.setExampleView(this);
    }

    private void initSomeActionButton()
    {
        someActionButton.addClickListener(event -> examplePresenter.update());
    }

    public void setValue(String value)
    {
        getModel().setValue(value);
    }

    @Override
    public void displayUpdate()
    {
        getUI().ifPresent(ui -> ui.access(() -> container.add(new CustomExample("Hello from update!"))));
    }
}
